#include <iostream>
#include <fstream>
#include <string>

#include <SFML/Graphics.hpp>

int retrieveInt(std::string &stringToParse) {
	std::string legalNumbers = "0123456789";
	size_t startPosition = stringToParse.find_first_of(legalNumbers, 0);
	size_t endPosition = stringToParse.find_first_not_of(legalNumbers, startPosition);
	int returnInt = atoi(stringToParse.substr(startPosition, endPosition).c_str());
	if(endPosition >= stringToParse.length())	{
		stringToParse = "";
	}
	else {
		stringToParse = stringToParse.substr(endPosition);
	}
	return returnInt;
}

struct SpriteItemGroup {
	std::string groupName;
	sf::Sprite groupSprite;
	std::vector<sf::IntRect> spriteRegions;
	int currentSprite;
	int animationDirection;
};

void nextAnimation(SpriteItemGroup &itemGroup) {
	if((itemGroup.currentSprite >= itemGroup.spriteRegions.size() - 1 && itemGroup.animationDirection == 1) || (itemGroup.currentSprite <= 0 && itemGroup.animationDirection == -1)) {
		itemGroup.animationDirection *= -1;
	}
	if(itemGroup.spriteRegions.size() > 1) {
		itemGroup.currentSprite += itemGroup.animationDirection;
	}
	itemGroup.groupSprite.SetSubRect(itemGroup.spriteRegions.at(itemGroup.currentSprite));
}

int main() {
	sf::RenderWindow display(sf::VideoMode(800, 600, 32), "Sprite Display");
	
	std::string spritesheetFilename = "";
	std::string parsingString = "";
	int startX = 0, startY = 0, endX = 0, endY = 0;
	
	int redTransparency = 0, greenTransparency = 0, blueTransparency = 0;
	std::vector<SpriteItemGroup> itemGroups;
	
	std::ifstream spritesheetDatafile;
	spritesheetDatafile.open("spritesheet.txt");
	
	if(spritesheetDatafile.is_open() && spritesheetDatafile.good()) {
		// Read in filename and transparency colors
		getline(spritesheetDatafile, spritesheetFilename);
		getline(spritesheetDatafile, parsingString);
		
		redTransparency = retrieveInt(parsingString);
		greenTransparency = retrieveInt(parsingString);
		blueTransparency = retrieveInt(parsingString);
		
		while(spritesheetDatafile.good()) {
			// Still can read groups
			SpriteItemGroup tempGroup;
			getline(spritesheetDatafile, tempGroup.groupName);
			tempGroup.currentSprite = 0;
			tempGroup.animationDirection = 1;
			getline(spritesheetDatafile, parsingString);
			while(parsingString.substr(0, 1) == " " || parsingString.substr(0,1) == "\t") {
				// Still have coordinates
				startX = retrieveInt(parsingString);
				startY = retrieveInt(parsingString);
				endX = retrieveInt(parsingString);
				endY = retrieveInt(parsingString);
				tempGroup.spriteRegions.push_back(sf::IntRect(startX, startY, endX, endY));
				getline(spritesheetDatafile, parsingString);
			}
			itemGroups.push_back(tempGroup);
		}
		
		spritesheetDatafile.close();
		
		sf::Image spritesheetImage;
		if(!spritesheetImage.LoadFromFile(spritesheetFilename)) {
			return EXIT_FAILURE;
		}
		// Setting transparency
		spritesheetImage.CreateMaskFromColor(sf::Color(redTransparency, greenTransparency, blueTransparency));
		
		for(int i = 0; i < itemGroups.size(); i++) {
			itemGroups.at(0).groupSprite.SetImage(spritesheetImage);
			
			if(itemGroups.at(i).spriteRegions.size() > 0) {
				itemGroups.at(i).groupSprite.SetSubRect(itemGroups.at(i).spriteRegions.at(0));
			}
		}
		
		itemGroups.at(0).groupSprite.SetPosition(250.0, 250.0);
		
		display.Clear(sf::Color(0, 255, 255));
		for(int i = 0; i < itemGroups.size(); i++) {
			display.Draw(itemGroups.at(i).groupSprite);
			nextAnimation(itemGroups.at(i));
		}
		display.Display();
		
		float elapsedTime = 0.0;
		sf::Clock gameClock;
		
		while(display.IsOpened()) {
			sf::Event event;
			while(display.GetEvent(event)) {
				if(event.Type == sf::Event::Closed) {
					display.Close();
				}
			}
			
			if(display.IsOpened()) {
				elapsedTime = gameClock.GetElapsedTime();
				// 15 FPS
				if(elapsedTime >= 1.0/15.0) {
					display.Clear(sf::Color(0, 255, 255));
					for(int i = 0; i < itemGroups.size(); i++) {
						display.Draw(itemGroups.at(i).groupSprite);
						nextAnimation(itemGroups.at(i));
					}
					display.Display();
					gameClock.Reset();
				}
			}
		}
	}
	
	if(display.IsOpened()) {
		display.Close();
	}
	
	return 0;
}
